@@ -6,19 +6,10 @@ use Tangram::Coll;
 
 package Tangram::AbstractSet;
 
-use vars qw(@ISA);
- @ISA = qw( Tangram::Coll );
+use base qw( Tangram::Coll );
 
 use Carp;
 
-# Support for classes that lazily create Set::Objects for instance vars.
-# -- ks.perl@kurtstephens.com 2004/03/30
-sub __lazy_members
-{
-  $_[0] ? $_[0]->members : ();
-}
-
-
 sub get_exporter
   {
 	my ($self, $context) = @_;
@@ -33,7 +24,7 @@ sub get_exporter
 		
 		my $storage = $context->{storage};
 		
-		foreach my $item ( __lazy_members($obj->{$field}) ) {
+		foreach my $item ($obj->{$field}->members) {
 		  $storage->_save($item, $context->{SAVING});
 		}
 		
@@ -49,16 +40,9 @@ sub get_exporter
 	  
 	  my $storage = $context->{storage};
 	  
-	  if (my $s = $obj->{$field}) {
-	      if (!UNIVERSAL::isa($s, "Set::Object")) {
-		  die "Data error in ${obj}"."->{$field}; expected "
-		      ."Set, got $s"
-	      } else {
-		  foreach my $item ( $s->members ) {
-		      $storage->insert($item)
-			  unless $storage->id($item);
-		  }
-	      }
+	  foreach my $item ($obj->{$field}->members) {
+		$storage->insert($item)
+		  unless $storage->id($item);
 	  }
 	  
 	  $storage->defer(sub { $self->defered_save(shift, $obj, $field, $self) } );
@@ -78,7 +62,7 @@ sub update
 	my $old_state = $old_states->{$member};
 	my %new_state = ();
 
-	foreach my $item ( __lazy_members($obj->{$member}) )
+	foreach my $item ($obj->{$member}->members)
 	{
 		my $item_id = $storage->id($item) || croak "member $item has no id";
       
@@ -105,19 +89,14 @@ sub remember_state
 	my ($self, $def, $storage, $obj, $member, $set) = @_;
 
 	my %new_state;
-	for my $member ( __lazy_members($set) ) {
-	    my $id = $storage->id($member);
-	    $id && ($new_state{ $id } = 1);
-	}
-
-	$storage->{scratch}{ref($self)}{$storage->id($obj)}{$member}
-	    = \%new_state;
+	@new_state{ map { $storage->id($_) } $set->members } = 1 x $set->size;
+	$storage->{scratch}{ref($self)}{$storage->id($obj)}{$member} = \%new_state;
 }
 
 sub content
 {
 	shift;
-	__lazy_members(shift); #?#?
+	shift->members;
 }
 
 1;
