@@ -5,8 +5,7 @@ use strict;
 package Tangram::Array;
 
 use Tangram::AbstractArray;
-use vars qw(@ISA);
- @ISA = qw( Tangram::AbstractArray );
+use base qw( Tangram::AbstractArray );
 
 use Carp;
 
@@ -115,7 +114,7 @@ sub cursor
 	$cursor->{-coll_tid} = $coll_tid;
 	$cursor->{-coll_cols} = "t$coll_tid.$slot_col";
 	$cursor->{-coll_from} = "$table t$coll_tid";
-	$cursor->{-coll_where} = "t$coll_tid.$coll_col = $coll_id AND t$coll_tid.$item_col = t$item_tid.$storage->{schema}{sql}{id_col}";
+	$cursor->{-coll_where} = "t$coll_tid.$coll_col = $coll_id AND t$coll_tid.$item_col = t$item_tid.id";
    
 	return $cursor;
 }
