@@ -14,7 +14,7 @@ sub new
 	confess unless $conn;
 
 	$remote = $storage->remote($remote)
-	    unless ref $remote or not defined $remote;
+	  unless ref $remote;
 
 	my $self = {};
 
@@ -64,44 +64,28 @@ sub select
 	}
 
 	$self->{-order} = $args{order};
-	$self->{-group} = $args{group};
 	$self->{-desc} = $args{desc};
 	$self->{-distinct} = $args{distinct};
 	$self->{-limit} = $args{limit};
 
-	my $objects = Set::Object->new();
-
-	if (exists $args{retrieve}) {
-	    $self->retrieve( @{ $args{retrieve} } );
-	    $objects->insert
-		( map { $_->{objects}->members }
-		  @{ $args{retrieve} } );
-	}
+	$self->retrieve( @{ $args{retrieve} } ) if exists $args{retrieve};
 
 	my $target = $self->{TARGET};
 
 	my (@filter_from, @filter_where);
 
-	my $filter = Tangram::Filter->new( tight => 100, objects => $objects );
+	my $filter = Tangram::Filter->new( tight => 100, objects => Set::Object->new() );
 
 	if (my $user_filter = $args{filter})
 	{
 		$filter->{expr} = $user_filter->{expr};
 		$filter->{objects}->insert($user_filter->{objects}->members);
-		$filter->{objects}->remove($target->object) if $target;
+		$filter->{objects}->remove($target->object);
 	}
 
-	my @polysel = 
-	     $self->{STORAGE}->get_polymorphic_select
-	     ( $target ? $target->class : "");
-
-	$self->{SELECTS} =
-	    [
-	     map {
-		 [ $self->build_select( $_, [], [ $filter->from ],
-					[ $filter->where ]), undef, $_ ]
-	     } @polysel
-	    ];
+	$self->{SELECTS} = [ map {
+	  [ $self->build_select( $_, [], [ $filter->from ],  [ $filter->where ]), undef, $_ ]
+	} $self->{STORAGE}->get_polymorphic_select( $target->class ) ];
 
 	$self->{position} = -1;
 
@@ -113,19 +97,15 @@ sub execute
 	my ($self) = @_;
 	return $self->{-current} if $self->{position} == 0;
 	$self->{cur_select} = [ @{ $self->{SELECTS} } ];
-	$self->prepare_next_statement() && $self->next();
+	return $self->prepare_next_statement() && $self->next();
   }
 
 sub prepare_next_statement
   {
 	my ($self) = @_;
 
-	my $select = $self->{ACTIVE} = shift @{ $self->{cur_select} }
-	    or do { 
-		#print $Tangram::TRACE "Cursor - no active selects?\n"
-		    #if $Tangram::TRACE;
-		return undef;
-	    };
+	my $select = $self->{ACTIVE} = shift @{ $self->{cur_select} } or return undef;
+
 	my ($sql, $sth, $template) = @$select;
 
 	$self->{sth}->finish() if $self->{sth};
@@ -135,7 +115,7 @@ sub prepare_next_statement
 
 	$self->{sth} = $sth;
 
-	$sth->execute() or croak "Execute failed; $DBI::errstr";
+	$sth->execute();
 
 	return $sth;
   }
@@ -154,40 +134,9 @@ sub build_select
 	substr($select, length('SELECT'), 0) = ' DISTINCT'
 	  if $self->{-distinct};
 
-	#unless ($self->{-my_rdbms_sucks} or !blessed $cols)
-	    #{
-		#$select .= ("\n\tGROUP BY ".$cols->root_table);
-	#}
-
-
-	if (my $group = $self->{-group})
-	{
-	   $DB::single = 1;
-           # add the fields that we're grouping by to the select part
-           # of the query
-           my @not_in_it = (grep { $select !~ m/ \Q$_\E(?:,|$)/ }
-			    map { $_->expr } @$group);
-           $select =~ s{\n}{join("", map {", $_"} @not_in_it)."\n"}se
-	       if @not_in_it;
-
-	   # FIXME - this should include *ALL* selected columns that
-	   # are not aggregate functions.  In short, normally you
-	   # would not want to select an object.
-           $select .= ("\n\tGROUP BY ".
-		       join ', ', map { $_->expr } @$group);
-	}
-
 	if (my $order = $self->{-order})
 	{
-           # add the fields that we're ordering by to the select part
-           # of the query
-           my @not_in_it = (grep { $select !~ m/ \Q$_\E(?:,|$)/ }
-			    map { $_->expr } @$order);
-           $select =~ s{\n}{join("", map {", $_"} @not_in_it)."\n"}se
-	       if @not_in_it;
-
-           $select .= ("\n\tORDER BY ".
-		       join ', ', map { $_->expr } @$order);
+		$select .= "\n\tORDER BY " . join ', ', map { $_->{expr} } @$order;
 	}
 
 	if ($self->{-desc})
@@ -195,13 +144,7 @@ sub build_select
 		$select .= ' DESC';
 	}
 
-	if (defined $self->{-limit}) {
-	    if (ref $self->{-limit}) {
-		$select .= " LIMIT ".join(",",@{ $self->{-limit} });
-	    } else {
-		$select .= " LIMIT $self->{-limit}";
-	    }
-	}
+	$select .= " LIMIT $self->{-limit}" if defined $self->{-limit};
 
 	return $select;
 }
@@ -213,9 +156,7 @@ sub _next
 	$self->{-current} = undef;
 	++$self->{position};
 
-	my $sth = $self->{sth}
-	    or confess "no sth";
-
+	my $sth = $self->{sth};
 	my @row;
 
 	while (1)
@@ -227,35 +168,25 @@ sub _next
 
 	my $storage = $self->{STORAGE};
 
-	if ($self->{TARGET}) {
-	    my ($id, $classId, $state) = $self->{ACTIVE}[-1]->extract(\@row);
+	my ($id, $classId, $state) = $self->{ACTIVE}[-1]->extract(\@row);
 
-	    $id = $storage->{import_id}->($id, $classId);
+	$id = $storage->{import_id}->($id, $classId);
 
-	    my $class = $storage->{id2class}{$classId} or die "unknown class id $classId";
+	my $class = $storage->{id2class}{$classId} or die "unknown class id $classId";
 
-	    # even if object is already loaded we must read it so that @rpw only contains residue
-	    my $obj = $storage->read_object($id, $class, $state);
-
-	    # if object is already loaded return previous copy
-	    $obj = $storage->{objects}{$id} if exists $storage->{objects}{$id};
-
-	    $self->{-current} = $obj;
-
-	} else {
-	    $self->{-current} = undef;
-	}
+	# even if object is already loaded we must read it so that @rpw only contains residue
+	my $obj = $storage->read_object($id, $class, $state);
 
 	$self->{-residue} = exists $self->{-retrieve}
-	    ? [ map { ref $_ ? $_->{type}->read_data(\@row) : shift @row } @{$self->{-retrieve}} ]
-		: \@row;
+		? [ map { ref $_ ? $_->{type}->read_data(\@row) : shift @row } @{$self->{-retrieve}} ]
+			: \@row;
 
-	$self->{-current} ||=
-	    (@{$self->{-residue}} > 1
-	     ? $self->{-residue}
-	     : $self->{-residue}[0]);
+	# if object is already loaded return previous copy
+	$obj = $storage->{objects}{$id} if exists $storage->{objects}{$id};
+   
+	$self->{-current} = $obj;
 
-	return $self->{-current};
+	return $obj;
 }
 
 sub next
