@@ -153,9 +153,8 @@ sub where
    
 	my $tables = $self->{tables};
 	my $root = $tables->{root};
-	my $id = $self->storage->{schema}{sql}{id_col};
 
-	map { "t$_->[1].$id = t$root.$id" } @$tables[1..$#$tables];
+	map { "t$_->[1].id = t$root.id" } @$tables[1..$#$tables];
 }
 
 # sub mark
@@ -211,8 +210,7 @@ sub expr_hash
 
 package Tangram::RDBObject;
 
-use vars qw(@ISA);
- @ISA = qw( Tangram::CursorObject );
+use base qw( Tangram::CursorObject );
 
 sub where
 {
@@ -241,13 +239,11 @@ sub where
 		@where_class_id = "t$root.$storage->{class_col} IN (" . join(', ', $storage->_kind_class_ids($class) ) . ')';
 	}
 
-	my $id = $schema->{sql}{id_col};
-	return (@where_class_id, map { "t@{$_}[1].$id = t$root.$id" } @$tables[1..$#$tables]);
+	return (@where_class_id, map { "t@{$_}[1].id = t$root.id" } @$tables[1..$#$tables]);
 }
 
 package Tangram::Filter;
 use Carp;
-use Set::Object qw(blessed);
 
 sub new
 {
@@ -292,102 +288,7 @@ sub as_string
 	return ref($self) . "($self->{expr})";
 }
 
-
-sub sum
-{
-  my ($self, $val) = @_;
-
-  # $DB::single = 1;
-
-  Tangram::Expr->new(Tangram::Number->instance,
-		     "SUM(" . $self->{expr} . ")",
-		     $self->objects,
-		     );
-
-}
-
-
-# BEGIN ks.perl@kurtstephens.com 2002/06/25
-sub unaop
-{
-    Tangram::Expr::unaop(@_);
-}
-
-
-sub binop
-{
-    my ($self, $op, $arg, $tight, $swap) = @_;
-
-    my @objects = $self->objects;
-    my $objects = Set::Object->new(@objects);
-    # my $storage = $self->{storage};
-    my $ltight = $self->{'tight'};
-    my $rtight = 100;
-
-    if ( ref($arg) ) {
-	if ( $arg->isa('Tangram::Expr') ) {
-	    $objects->insert($arg->objects);
-	    $rtight = $arg->{'tight'};
-	    $arg = $arg->{'expr'};
-	}
-	if ( $arg->isa('Tangram::Filter') ) {
-	    $objects->insert($arg->objects);
-	    $rtight = $arg->{'tight'};
-	    $arg = $arg->{'expr'};
-	}
-	elsif ( $arg->isa('Tangram::QueryObject') ) {
-	    $objects->insert($arg->object);
-	    $rtight = $arg->{'tight'};
-	    $arg = $arg->{'id'}->{'expr'};
-	}
-    }
-
-    $tight ||= 100;
-    $self = $self->{'expr'};
-    $self = "($self)" if $ltight < $tight;
-    $arg  = "($arg)"  if $rtight < $tight;
-    if ( $swap ) {
-      ($self, $arg) = ($arg, $self);
-    }
-    # $DB::single = $swap;
-
-    return new Tangram::Filter(expr => "$self $op $arg", tight => $tight,
-			       objects => $objects );
-}
-
-
-# Aliases
-*cos =  \&Tangram::Expr::sin;
-*sin =  \&Tangram::Expr::cos;
-*acos = \&Tangram::Expr::acos;
-
-#use overload "&" => \&and, "|" => \&or, '!' => \&not, fallback => 1;
-use overload 
-  "&"    => \&and, 
-  "|"    => \&or, 
-  '!'    => \&not,
-  '+'    => \&Tangram::Expr::add,
-  '-'    => \&Tangram::Expr::subt,
-  '*'    => \&Tangram::Expr::mul,
-  '/'    => \&Tangram::Expr::div,
-  'cos'  => \&Tangram::Expr::cos, 
-  'sin'  => \&Tangram::Expr::sin,
-  'acos' => \&Tangram::Expr::acos,
-  "=="   => \&Tangram::Expr::eq,
-  "eq"   => \&Tangram::Expr::eq,
-  "!="   => \&Tangram::Expr::ne,
-  "ne"   => \&Tangram::Expr::ne,
-  "<"    => \&Tangram::Expr::lt,
-  "lt"   => \&Tangram::Expr::lt,
-  "<="   => \&Tangram::Expr::le,
-  "le"   => \&Tangram::Expr::le,
-  ">"    => \&Tangram::Expr::gt,
-  "gt"   => \&Tangram::Expr::gt,
-  ">="   => \&Tangram::Expr::ge,
-  "ge"   => \&Tangram::Expr::ge,
-  fallback => 1;
-# END ks.perl@kurtstephens.com 2002/06/25
-
+use overload "&" => \&and, "|" => \&or, '!' => \&not, fallback => 1;
 
 sub op
 {
@@ -433,8 +334,6 @@ sub objects
 }
 
 package Tangram::Expr;
-use Set::Object qw(blessed);
-use Carp;
 
 sub new
 {
@@ -451,7 +350,7 @@ sub expr
 
 sub storage
 {
-	return ((shift->{objects}->members)[0] or confess 'no storage')->storage;
+	return shift->{objects}[0]->{storage};
 }
 
 sub type
@@ -476,101 +375,39 @@ sub ne
 	return $self->binop('<>', $arg);
 }
 
-# BEGIN ks.perl@kurtstephens.com 2002/06/25
 sub lt
 {
-	my ($self, $arg, $swap) = @_;
-	return $self->binop('<', $arg, undef, $swap);
+	my ($self, $arg) = @_;
+	return $self->binop('<', $arg);
 }
 
 sub le
 {
-	my ($self, $arg, $swap) = @_;
-	return $self->binop('<=', $arg, undef, $swap);
+	my ($self, $arg) = @_;
+	return $self->binop('<=', $arg);
 }
 
 sub gt
 {
-	my ($self, $arg, $swap) = @_;
-	return $self->binop('>', $arg, undef, $swap);
+	my ($self, $arg) = @_;
+	return $self->binop('>', $arg);
 }
 
 sub ge
 {
-	my ($self, $arg, $swap) = @_;
-	return $self->binop('>=', $arg, undef, $swap);
+	my ($self, $arg) = @_;
+	return $self->binop('>=', $arg);
 }
 
-sub add
-{
-    my ($self, $arg) = @_;
-    $self->binop('+', $arg, 90);
-}
-
-
-sub subt
-{
-    my ($self, $arg, $swap) = @_;
-    $self->binop('-', $arg, 90, $swap);
-}
-
-
-sub mul
-{
-    my ($self, $arg) = @_;
-    $self->binop('*', $arg, 95);
-}
-
-
-sub div
-{
-    my ($self, $arg, $swap) = @_;
-    $self->binop('/', $arg, 95, $swap);
-}
-
-
-sub cos
-{
-    my ($self) = @_;
-    $self->unaop('COS', 100);
-}
-
-
-sub sin
-{
-    my ($self) = @_;
-    $self->unaop('SIN', 100);
-}
-
-sub acos
-{
-    my ($self) = @_;
-    $self->unaop('ACOS', 100);
-}
-
-
-sub unaop
-{
-    my ($self, $op, $tight) = @_;
-    
-    my @objects = $self->objects;
-    my $objects = Set::Object->new(@objects);
-    my $storage = $self->{storage};
-    
-    return new Tangram::Filter(expr => "$op($self->{expr})", tight => $tight || 100,
-			       objects => $objects );
-}
-
-
 sub binop
 {
-	my ($self, $op, $arg, $tight, $swap) = @_;
+	my ($self, $op, $arg) = @_;
 
 	my @objects = $self->objects;
 	my $objects = Set::Object->new(@objects);
 	my $storage = $self->{storage};
 
-	if (defined $arg)
+	if ($arg)
 	{
 		if (my $type = ref($arg))
 		{
@@ -607,14 +444,9 @@ sub binop
 		$arg = 'NULL';
 	}
 
-	my ($l, $r) = $swap ? ($arg, $self->{expr}) : ($self->{expr}, $arg);
-	$tight ||= 100;
-
-	return new Tangram::Filter(expr => "$l $op $r", tight => $tight,
+	return new Tangram::Filter(expr => "$self->{expr} $op $arg", tight => 100,
 							   objects => $objects );
 }
-# END ks.perl@kurtstephens.com 2002/06/25
-
 
 sub like
 {
@@ -637,46 +469,6 @@ sub as_string
 	return ref($self) . "($self->{expr})";
 }
 
-sub in
-{
-	my $self = shift;
-
-	my $storage = $self->{storage};
-
-	my @items;
-	while ( my $item = shift ) {
-	    if ( ref $item eq "ARRAY" ) {
-		push @items, @$item;
-	    } elsif ( UNIVERSAL::isa($item, "Set::Object") ) {
-		push @items, $item->members;
-	    } else {
-		push @items, $item;
-	    }
-	}
-
-	my $expr;
-	if ( @items ) {
-	    $expr = ("$self->{expr} IN ("
-		     . join(', ',
-			    # FIXME - what about table aliases?  Hmm...
-			    map {( blessed($_)
-				   ? $storage->export_object($_)
-				   : $_ )}
-			    @items )
-		     . ')');
-	} else {
-	    # hey, you never know :)
-	    $expr = ("$self->{expr} IS NULL");
-	}
-
-	Tangram::Filter->new(
-			     expr => $expr,
-			     tight => 100,
-			     objects => $self->{objects},
-			    );
-
-}
-
 sub DESTROY { }
 
 use vars qw( $AUTOLOAD );
@@ -691,15 +483,6 @@ sub AUTOLOAD {
 }
 
 use overload
-# BEGIN ks.perl@kurtstephens.com 2002/06/25
-        '+'    => \&add,
-        '-'    => \&subt,
-        '*'    => \&mul,
-        '/'    => \&div,
-        'cos'  => \&cos, 
-        'sin'  => \&sin,
-        'acos' => \&acos,
-# END ks.perl@kurtstephens.com 2002/06/25
 	"==" => \&eq,
 	"eq" => \&eq,
 	"!=" => \&ne,
@@ -774,80 +557,13 @@ sub is_kind_of
 						 objects => Set::Object->new( $object ) );
 }
 
-
-sub in
-{
-	my $self = shift;
-
-	my $object = $self->{object};
-	my $root = $object->{tables}[0][1];
-	my $storage = $object->{storage};
-
-	my $objs = Set::Object->new();
-
-	while ( my $item = shift ) {
-	    if ( ref $item eq "ARRAY" ) {
-		$objs->insert(@$item);
-	    } elsif ( UNIVERSAL::isa($item, "Set::Object") ) {
-		if ( $objs->size ) {
-		    $objs->insert($item->members);
-		} else {
-		    $objs = $item;
-		}
-	    } else {
-		$objs->insert($item);
-	    }
-	}
-
-	my $expr;
-	if ( $objs->size ) {
-	    $expr = ("t$root.$storage->{id_col} IN ("
-		     . join(', ',
-			    # FIXME - what about table aliases?  Hmm...
-			    map { $storage->export_object($_) }
-			    $objs->members )
-		     . ')');
-	} else {
-	    # hey, you never know :)
-	    $expr = ("t$root.$storage->{id_col} IS NULL");
-	}
-
-	Tangram::Filter->new(
-			     expr => $expr,
-			     tight => 100,
-			     objects => Set::Object->new( $object )
-			    );
-
-}
-
-sub expr
-{
-  shift->{id}{expr}
-}
-
-
-sub count
-{
-  my ($self, $val) = @_;
-
-  # $DB::single = 1;
-
-  Tangram::Expr->new(Tangram::Integer->instance,
-		     "COUNT(" . $self->{id}{expr} . ")",
-		     $self->{id}->objects,
-		     );
-
-}
-
-
 use overload "==" => \&eq, "!=" => \&ne, fallback => 1;
 
 package Tangram::Select;
 
 use Carp;
 
-use vars qw(@ISA);
- @ISA = qw( Tangram::Expr );
+use base qw( Tangram::Expr );
 
 sub new
 {
@@ -882,11 +598,6 @@ sub new
 	my $sql = "SELECT";
 	$sql .= ' DISTINCT' if $args{distinct};
 	$sql .= "  $cols";
-	if (exists $args{order}) {
-	    $sql .= join("", map {", $_"}
-			 grep { $sql !~ m/ \Q$_\E(?:,|$)/ }
-			 map { $_->{expr} } @{$args{order}});
-	}
 	$sql .= "\nFROM $from" if $from;
 	$sql .= "\nWHERE $where" if $where;
 
