@@ -23,8 +23,6 @@ sub includes
 	my ($self, $item) = @_;
 	my ($coll, $memdef) = @$self;
 
-	my $schema = $coll->{storage}{schema};
-
 	$item = quote($item)
 	  if $memdef->{string_type};
 
@@ -33,7 +31,7 @@ sub includes
 
 	return Tangram::Filter->new
 		(
-		 expr => "$data_tid.coll = $coll_tid.$schema->{sql}{id_col} AND $data_tid.v = $item",
+		 expr => "$data_tid.coll = $coll_tid.id AND $data_tid.v = $item",
 		 tight => 100,      
 		 objects => Set::Object->new($coll, Tangram::Table->new($memdef->{table}, $data_tid) ),
 		 data_tid => $data_tid # for prefetch
@@ -45,8 +43,6 @@ sub exists
 	my ($self, $item) = @_;
 	my ($coll, $memdef) = @$self;
 
-	my $schema = $coll->{storage}{schema};
-
 	$item = quote($item)
 	  if $memdef->{string_type};
 
@@ -54,15 +50,14 @@ sub exists
 
 	return Tangram::Filter->new
 		(
-		 expr => "EXISTS (SELECT * FROM $memdef->{table} WHERE coll = $coll_tid.$schema->{sql}{id_col} AND v = $item)",
+		 expr => "EXISTS (SELECT * FROM $memdef->{table} WHERE coll = $coll_tid.id AND v = $item)",
 		 objects => Set::Object->new($coll),
 		);
 }
 
 package Tangram::FlatArray;
 
-use vars qw(@ISA);
- @ISA = qw( Tangram::AbstractArray );
+use base qw( Tangram::AbstractArray );
 use Tangram::AbstractArray;
 
 $Tangram::Schema::TYPES{flat_array} = Tangram::FlatArray->new;
@@ -190,7 +185,8 @@ sub erase
 
 	foreach my $def (values %$members)
 	{
-		$storage->sql_do("DELETE FROM $def->{table} WHERE coll = $coll_id");
+		my $id = $storage->id($obj);
+		$storage->sql_do("DELETE FROM $def->{table} WHERE coll = $id");
 	}
 }
 
