@@ -15,8 +15,6 @@ sub includes
 	my ($self, $item) = @_;
 	my ($coll, $memdef) = @$self;
 
-	my $schema = $coll->{storage}{schema};
-
 	$item = Tangram::String::quote($item)
 		if $memdef->{string_type};
 
@@ -25,7 +23,7 @@ sub includes
 
 	return Tangram::Filter->new
 		(
-		 expr => "$data_tid.coll = $coll_tid.$schema->{sql}{id_col} AND $data_tid.v = $item",
+		 expr => "$data_tid.coll = $coll_tid.id AND $data_tid.v = $item",
 		 tight => 100,      
 		 objects => Set::Object->new($coll, Tangram::Table->new($memdef->{table}, $data_tid) ),
 		 data_tid => $data_tid # for prefetch
@@ -37,8 +35,6 @@ sub exists
 	my ($self, $item) = @_;
 	my ($coll, $memdef) = @$self;
 
-	my $schema = $coll->{storage}{schema};
-
 	$item = Tangram::String::quote($item)
 		if $memdef->{string_type};
 
@@ -46,15 +42,14 @@ sub exists
 
 	return Tangram::Filter->new
 		(
-		 expr => "EXISTS (SELECT * FROM $memdef->{table} WHERE coll = $coll_tid.$schema->{sql}{id_col} AND v = $item)",
+		 expr => "EXISTS (SELECT * FROM $memdef->{table} WHERE coll = $coll_tid.id AND v = $item)",
 		 objects => Set::Object->new($coll),
 		);
 }
 
 package Tangram::FlatHash;
 
-use vars qw(@ISA);
- @ISA = qw( Tangram::AbstractHash );
+use base qw( Tangram::AbstractHash );
 use Tangram::AbstractHash;
 
 $Tangram::Schema::TYPES{flat_hash} = Tangram::FlatHash->new;
@@ -262,7 +257,8 @@ sub erase
 
 	foreach my $def (values %$members)
 	{
-		$storage->sql_do("DELETE FROM $def->{table} WHERE coll = $coll_id");
+		my $id = $storage->id($obj);
+		$storage->sql_do("DELETE FROM $def->{table} WHERE coll = $id");
 	}
 }
 
