@@ -5,43 +5,43 @@ use strict;
 package Tangram::IntrArray;
 
 use Tangram::AbstractArray;
-use vars qw(@ISA);
- @ISA = qw( Tangram::AbstractArray );
+use base qw( Tangram::AbstractArray );
 
 use Carp;
 
-sub reschema {
-    my ($self, $members, $class, $schema) = @_;
+sub reschema
+{
+	my ($self, $members, $class, $schema) = @_;
 
-    foreach my $member (keys %$members) {
-	my $def = $members->{$member};
+	foreach my $member (keys %$members)
+	{
+		my $def = $members->{$member};
 
-	unless (ref($def))
-	    {
-		$def = { class => $def };
-		$members->{$member} = $def;
-	    }
+		unless (ref($def))
+		{
+			$def = { class => $def };
+			$members->{$member} = $def;
+		}
 
-	$def->{coll} ||= ($schema->{normalize}->($class)
-			  . "_$member");
-	$def->{slot} ||= ($schema->{normalize}->($class)
-			  . "_$member" . "_slot");
+		$def->{coll} ||= $class . "_$member";
+		$def->{slot} ||= $class . "_$member" . "_slot";
    
-	$schema->{classes}{$def->{class}}{stateless} = 0;
+		$schema->{classes}{$def->{class}}{stateless} = 0;
 
-	if (exists $def->{back}) {
-	    my $back = $def->{back} ||= $def->{item};
-	    $schema->{classes}{ $def->{class} }{members}{backref}{$back} =
-		bless {
-		       name => $back,
-		       col => $def->{coll},
-		       class => $class,
-		       field => $member
-		      }, 'Tangram::BackRef';
+		if (exists $def->{back})
+		{
+			my $back = $def->{back} ||= $def->{item};
+			$schema->{classes}{ $def->{class} }{members}{backref}{$back} =
+			  bless {
+					 name => $back,
+					 col => $def->{coll},
+					 class => $class,
+					 field => $member
+					}, 'Tangram::BackRef';
+		}
 	}
-    }
 
-    return keys %$members;
+	return keys %$members;
 }
 
 sub defered_save
@@ -60,6 +60,7 @@ sub defered_save
 	my $coll_id = $storage->export_object($obj);
 	
 	my $classes = $storage->{schema}{classes};
+	#use Data::Dumper; print Dumper \@_;
 	my $item_classdef = $classes->{ $def->{class} };
 	my $table = $item_classdef->{table} or die;
 	my $item_col = $def->{coll};
@@ -72,8 +73,6 @@ sub defered_save
 	
 	my $old_state = $self->get_load_state($storage, $obj, $field) || [];
 	my $old_size = $old_state ? @$old_state : 0;
-	# FIXME - where on earth are the undef values coming from ?  :(
-	@$old_state = grep { defined } @$old_state;
 	
 	my %removed;
 	@removed{ @$old_state } = () if $old_state;
@@ -85,11 +84,7 @@ sub defered_save
 		my $item_id = $storage->id( $coll->[$slot] ) || die;
 		my $ex_item_id = $storage->{export_id}->($item_id);
 		
-		$storage->sql_do
-		    ("UPDATE $table SET "
-		     ."$item_col = $coll_id, "
-		     ."$slot_col = $slot WHERE "
-		     ."$storage->{schema}{sql}{id_col} = $ex_item_id")
+		$storage->sql_do("UPDATE $table SET $item_col = $coll_id, $slot_col = $slot WHERE id = $ex_item_id")
 		  unless $slot < $old_size && $item_id eq $old_state->[$slot];
 		
 		push @new_state, $item_id;
@@ -100,7 +95,7 @@ sub defered_save
 	if (keys %removed)
 	  {
 		my $removed = join(', ', map { $storage->{export_id}->($_) } keys %removed);
-		$storage->sql_do("UPDATE $table SET $item_col = NULL, $slot_col = NULL WHERE $storage->{schema}{sql}{id_col} IN ($removed)");
+		$storage->sql_do("UPDATE $table SET $item_col = NULL, $slot_col = NULL WHERE id IN ($removed)");
 	  }
 	
 	$self->set_load_state($storage, $obj, $field, \@new_state);	
@@ -124,7 +119,7 @@ sub erase
 		}
 		else
 		{
-			my $item_classdef = $storage->{schema}{classes}{$def->{class}};
+			my $item_classdef = $storage->{schema}{$def->{class}};
 			my $table = $item_classdef->{table} || $def->{class};
 			my $item_col = $def->{coll};
 			my $slot_col = $def->{slot};
@@ -168,14 +163,22 @@ sub prefetch
 	my ($self, $storage, $def, $coll, $class, $member, $filter) = @_;
 
 	my $ritem = $storage->remote($def->{class});
+
+	my $ids = $storage->my_select_data( cols => [ $coll->{id} ], filter => $filter );
 	my $prefetch = $storage->{PREFETCH}{$class}{$member} ||= {}; # weakref
 
+	while (my ($id) = $ids->fetchrow)
+	{
+		$prefetch->{$id} = []
+	}
+
 	my $cursor = Tangram::Cursor->new($storage, $ritem, $storage->{db});
 	
 	my $includes = $coll->{$member}->includes($ritem);
 	$includes &= $filter if $filter;
 
 	# also retrieve collection-side id and index of elmt in sequence
+
 	$cursor->retrieve($coll->{id},
 	    $storage->expr(Tangram::Integer->instance,
 			"t$ritem->{object}{table_hash}{$def->{class}}.$def->{slot}") );
@@ -185,7 +188,7 @@ sub prefetch
 	while (my $item = $cursor->current)
 	{
 		my ($coll_id, $slot) = $cursor->residue;
-		($prefetch->{$coll_id}||=[])->[$slot] = $item;
+		$prefetch->{$coll_id}[$slot] = $item;
 		$cursor->next;
 	}
 
