@@ -6,12 +6,11 @@ use Tangram::Scalar;
 
 package Tangram::RawDateTime;
 
-use vars qw(@ISA);
- @ISA = qw( Tangram::String );
+use base qw( Tangram::String );
 
 $Tangram::Schema::TYPES{rawdatetime} = Tangram::RawDateTime->new;
 
-sub coldefs
+sub Tangram::RawDateTime::coldefs
 {
     my ($self, $cols, $members, $schema) = @_;
     $self->_coldefs($cols, $members, "DATETIME $schema->{sql}{default_null}");
