@@ -12,23 +12,32 @@ sub class
    $self->{$class} or croak "unknown class '$class'";
 }
 
-package Tangram::Node;
+package Tangram::Class;
 
-sub get_bases
+sub members
+{
+   my ($self, $type) = @_;
+   return @{$self->{$type}};
+}
+
+sub direct_bases
   {
 	@{ shift->{BASES} }
   }
 
-*direct_bases = \&get_bases;
-
-sub get_specs
+sub is_root
   {
-	@{ shift->{SPECS} }
+	!@{ shift->{BASES} }
+  }
+
+sub direct_fields
+  {
+	map { values %$_ } values %{ shift->{fields} }
   }
 
 sub for_conforming
 {
-   my ($class, $fun, @args) = @_;
+   my ($class, $fun) = @_;
    my $done = Set::Object->new;
 
    my $traverse;
@@ -37,7 +46,7 @@ sub for_conforming
 	 my $class = shift;
 	 return if $done->includes($class);
 	 $done->insert($class);
-	 $fun->($class, @args);
+	 $fun->($class);
 
 	 foreach my $derived (@{ $class->{SPECS} }) {
 	   $traverse->($derived);
@@ -47,157 +56,8 @@ sub for_conforming
    $traverse->($class);
  }
 
-#---------------------------------------------------------------------
-#  Tangram::Node->for_composing($closure, @_)
-#
-# Runs the given closure once for this class, and all its superclasses
-# listed in the schema as $class->{BASES}
-#
-#---------------------------------------------------------------------
-sub for_composing
-{
-   my ($class, $fun, @args) = @_;
-   my $done = Set::Object->new;
-
-   my $traverse;
-
-   $traverse = sub {
-	 my $class = shift;
-	 return if $done->includes($class);
-	 $done->insert($class);
-
-	 foreach my $base (@{ $class->{BASES} }) {
-	   $traverse->($base);
-	 }
-
-	 $fun->($class, @args);
-   };
-
-   $traverse->($class);
- }
-
-sub get_exporter {
-  my ($self, $context) = @_;
-
-  return $self->{EXPORTER} ||= do {
-
-	my (@export_sources, @export_closures);
-
-	$self->for_composing
-	    ( sub {
-		  my ($part) = @_;
-
-		  $context->{class} = $part;
-
-		  for my $field ($part->direct_fields()) {
-		      if (my $exporter = $field->get_exporter($context)) {
-			  if (ref $exporter) {
-			      push @export_closures, $exporter;
-			      push @export_sources, 'shift(@closures)->($obj, $context)';
-			  } else {
-			      push @export_sources, $exporter;
-			  }
-		      }
-		  }
-	      } );
-
-	my $export_source = join ",\n", @export_sources;
-	my $copy_closures =
-	    ( @export_closures ? ' my @closures = @export_closures;' : '' );
-
-	# $Tangram::TRACE = \*STDOUT;
-
-	$export_source = ("sub { my (\$obj, \$context) = \@_;"
-			  ."$copy_closures\n$export_source }");
-
-	print $Tangram::TRACE "Compiling exporter for $self->{name}...\n".($Tangram::DEBUG_LEVEL > 1 ? "$export_source\n" : "")
-	    if $Tangram::TRACE;
-
-	eval $export_source or die;
-    }
-}
-
-sub get_importer {
-  my ($self, $context) = @_;
-
-  return $self->{IMPORTER} ||= do {
-	my (@import_sources, @import_closures);
-
-	$self->for_composing
-	    ( sub {
-		  my ($part) = @_;
-
-		  $context->{class} = $part;
-
-		  for my $field ($part->get_direct_fields()) {
-
-		      my $importer = $field->get_importer($context)
-			  or next;
-
-		      if (ref $importer) {
-			  push @import_closures, $importer;
-			  push @import_sources, 'shift(@closures)->($obj, $row, $context)';
-		      } else {
-			  push @import_sources, $importer;
-		      }
-		  }
-	      } );
-
-	my $import_source = join ";\n", @import_sources;
-	my $copy_closures =
-	    ( @import_closures ? ' my @closures = @import_closures;' : '' );
-
-	# $Tangram::TRACE = \*STDOUT;
-
-	$import_source = ( "sub { my (\$obj, \$row, \$context) = \@_;"
-			   ."$copy_closures\n$import_source }" );
-
-	print $Tangram::TRACE "Compiling importer for $self->{name}...\n".($Tangram::DEBUG_LEVEL > 1 ? "$import_source\n" : "")."\n"
-	  if $Tangram::TRACE;
-
-	# use Data::Dumper; print Dumper \@cols;
-	eval $import_source or die;
-  };
-}
-
-package Tangram::Class;
-use vars qw(@ISA);
- @ISA = qw( Tangram::Node );
-
-sub members
-{
-   my ($self, $type) = @_;
-   return @{$self->{$type}};
-}
-
-sub is_root
-  {
-	!@{ shift->{BASES} }
-  }
-
-sub get_direct_fields
-  {
-	map { values %$_ } values %{ shift->{fields} }
-  }
-
-sub get_table { shift->{table} }
-
-*direct_fields = \&get_direct_fields;
-
-sub get_import_cols {
-  my ($self, $context) = @_;
-  my $table = $self->{table};
-  map { map { [ $table, $_ ] } $_->get_import_cols($context) } $self->get_direct_fields()
-}
-
-sub get_export_cols {
-  my ($self, $context) = @_;
-  my $table = $self->{table};
-  map { map { [ $table, $_ ] } $_->get_export_cols($context) } $self->get_direct_fields()
-}
-
 package Tangram::Schema;
-#our @ISA = qw( SelfLoader );
+#use base qw( SelfLoader );
 use Carp;
 
 use vars qw( %TYPES );
@@ -205,15 +65,14 @@ use vars qw( %TYPES );
 %TYPES = 
 (
    %TYPES,
-#   ref      => new Tangram::Ref,
+   ref      => new Tangram::Ref,
 );
 
 sub new
 {
     my $pkg = shift;
 
-    my $self = ref $_[0] ? shift() : { @_ };
-
+	my $self = ref $_[0] ? shift() : { @_ };
     bless $self, $pkg;
 
     $self->{make_object} ||= sub { shift()->new() };
@@ -221,24 +80,21 @@ sub new
     $self->{normalize} ||= sub { shift() };
     $self->{class_table} ||= 'OpalClass';
 
-    $self->{control} ||= 'Tangram';
+	$self->{control} ||= 'Tangram';
 
-    $self->{sql}{default_null} = 'NULL' unless exists $self->{sql}{default_null};
-    $self->{sql}{id_col} ||= 'id';
-    $self->{sql}{id} ||= 'INTEGER';
-    # commented out because of layout1 compatibility $self->{sql}{class_col} ||= 'type';
-    $self->{sql}{cid} ||= 'INTEGER';
-    $self->{sql}{oid} ||= 'INTEGER';
-    $self->{sql}{cid_size} ||= 4;
-
-    $self->{sql}{dumper} ||= "Storable";
-    $self->{sql}{dumper_type} ||= "BLOB";
+	$self->{sql}{default_null} = 'NULL' unless exists $self->{sql}{default_null};
+	$self->{sql}{id_col} ||= 'id';
+	$self->{sql}{id} ||= 'INTEGER';
+	# commented out because of layout1 compatibility $self->{sql}{class_col} ||= 'type';
+	$self->{sql}{cid} ||= 'INTEGER';
+	$self->{sql}{oid} ||= 'INTEGER';
+	$self->{sql}{cid_size} ||= 4;
 
     my $types = $self->{types} ||= {};
 
     %$types = ( %TYPES, %$types );
 
-    my @class_list = ref($self->{'classes'}) eq 'HASH' ? %{ $self->{'classes'} } : @{ $self->{'classes'} };
+	my @class_list = ref($self->{'classes'}) eq 'HASH' ? %{ $self->{'classes'} } : @{ $self->{'classes'} };
     my $class_hash = $self->{'classes'} = {};
 
     bless $class_hash, 'Tangram::ClassHash';
@@ -262,28 +118,6 @@ sub new
 		$classdef->{table} ||= $self->{normalize}->($class, 'tablename');
 
 		$classdef->{fields} ||= $classdef->{members};
-
-		if ( $classdef->{members} and
-		     $classdef->{fields} != $classdef->{members} ) {
-		    # some other class' definition put something
-		    # in our "members" hash.
-		    while (
-			   my ($type, $fields)
-			   = each %{$classdef->{members}}
-			  )
-		    {
-			# so, we have to merge them.  we could use
-			#  %{ $classdef->{fields} } =
-			#    (%{$classdef->{fields}},
-			#     %{$classdef->{members}});
-			# but I'm not 100% sure that we will never
-			# have common types in the "fields" and
-			# "members" hash.
-			($classdef->{fields}{$type}{$_}
-			 = delete $fields->{$_})
-			    foreach (keys %$fields);
-		    }
-		}
 		$classdef->{members} = $classdef->{fields};
 
 		my $cols = 0;
@@ -291,34 +125,30 @@ sub new
 		foreach my $typetag (keys %{$classdef->{members}})
 		{
 			my $memdefs = $classdef->{members}{$typetag};
-
-			# Aha, so *here* is where the array is reschema'd.
-			$memdefs = $classdef->{members}{$typetag}
-			    = { map { $_, $_ } @$memdefs }
-				if (ref $memdefs eq 'ARRAY');
+	    
+			$memdefs = $classdef->{members}{$typetag} =
+			{ map { $_, $_ } @$memdefs } if (ref $memdefs eq 'ARRAY');
 
 			my $type = $self->{types}{$typetag};
 
-			croak("Unknown field type '$typetag', ",
-			      "did you forget some 'use Tangram::SomeType' ",
-			      "in your program?\n")
-			    unless defined $types->{$typetag};
+			croak("Unknow field type '$typetag', ",
+				  "did you forget some 'use Tangram::SomeType' ",
+				  "in your program?\n")
+				unless defined $types->{$typetag};
 
-			my @members = $types->{$typetag}->reschema
-			    ($memdefs, $class, $self)
+			my @members = $types->{$typetag}->reschema($memdefs, $class, $self)
 				if $memdefs;
 
 			for my $field (keys %$memdefs) {
-			    $memdefs->{$field}{name} = $field;
-			    my $fielddef = bless $memdefs->{$field}, ref $type;
-			    my @cols = $fielddef->get_export_cols( {} );
-			    $cols += @cols;
+			  $memdefs->{$field}{name} = $field;
+			  my $fielddef = bless $memdefs->{$field}, ref $type;
+			  my @cols = $fielddef->get_export_cols( {} );
+			  $cols += @cols;
 			}
 
-			@{$classdef->{member_type}}{@members}
-			    = ($type) x @members;
-			@{$classdef->{MEMDEFS}}{keys %$memdefs}
-			    = values %$memdefs;
+			@{$classdef->{member_type}}{@members} = ($type) x @members;
+	    
+			@{$classdef->{MEMDEFS}}{keys %$memdefs} = values %$memdefs;
 		}
 
 		$classdef->{stateless} = !$cols
@@ -372,8 +202,6 @@ sub classdef
    return $self->{classes}{$class} or confess "unknown class '$class'";
 }
 
-*get_class_by_name = \&classdef;
-
 sub classes
 {
    my ($self) = @_;
@@ -580,3 +408,5 @@ sub is_persistent
 
 1;
 
+1;
+
